@@ -4,7 +4,6 @@ import locale
 import sys
 from contextlib import contextmanager
 from datetime import timedelta
-from os.path import expanduser
 from os.path import isdir
 
 import click
@@ -15,10 +14,9 @@ from todoman import formatters
 from todoman.configuration import ConfigurationException
 from todoman.configuration import load_config
 from todoman.interactive import TodoEditor
-from todoman.model import cached_property
 from todoman.model import Database
 from todoman.model import Todo
-
+from todoman.model import cached_property
 
 click_log.basic_config()
 
@@ -52,8 +50,8 @@ def _validate_lists_param(ctx, param=None, lists=()):
 def _validate_list_param(ctx, param=None, name=None):
     ctx = ctx.find_object(AppContext)
     if name is None:
-        if ctx.config["main"]["default_list"]:
-            name = ctx.config["main"]["default_list"]
+        if ctx.config["default_list"]:
+            name = ctx.config["default_list"]
         else:
             raise click.BadParameter("You must set `default_list` or use -l.")
     lists = {list_.name: list_ for list_ in ctx.db.lists()}
@@ -110,7 +108,7 @@ def _validate_start_date_param(ctx, param, val):
 
 def _validate_startable_param(ctx, param, val):
     ctx = ctx.find_object(AppContext)
-    return val or ctx.config["main"]["startable"]
+    return val or ctx.config["startable"]
 
 
 def _validate_todos(ctx, param, val):
@@ -126,22 +124,16 @@ def _sort_callback(ctx, param, val):
             field = field[1:]
 
         if field not in Todo.ALL_SUPPORTED_FIELDS and field != "id":
-            raise click.BadParameter("Unknown field '{}'".format(field))
+            raise click.BadParameter(f"Unknown field '{field}'")
 
     return fields
 
 
-def validate_status(ctx=None, param=None, val=None):
-    # The default command doesn't run callbacks as expected, so it needs to
-    # specify the callback'd type. When `list` is called explicitly, this
-    # callback *IS* run, so we need to handle that edge case:
-    if not isinstance(val, str):
-        return val
-
+def validate_status(ctx=None, param=None, val=None) -> str:
     statuses = val.upper().split(",")
 
     if "ANY" in statuses:
-        return Todo.VALID_STATUSES
+        return ",".join(Todo.VALID_STATUSES)
 
     for status in statuses:
         if status not in Todo.VALID_STATUSES:
@@ -151,7 +143,7 @@ def validate_status(ctx=None, param=None, val=None):
                 )
             )
 
-    return statuses
+    return val
 
 
 def _todo_property_options(command):
@@ -159,7 +151,7 @@ def _todo_property_options(command):
         "--priority",
         default="",
         callback=_validate_priority_param,
-        help="The priority for this todo",
+        help="Priority for this task",
     )(command)
     click.option("--location", help="The location where this todo takes place.")(
         command
@@ -169,10 +161,7 @@ def _todo_property_options(command):
         "-d",
         default="",
         callback=_validate_date_param,
-        help=(
-            "The due date of the task, in the format specified in the "
-            "configuration file."
-        ),
+        help=("Due date of the task, in the format specified in the configuration."),
     )(command)
     click.option(
         "--start",
@@ -201,17 +190,17 @@ class AppContext:
     @cached_property
     def ui_formatter(self):
         return formatters.DefaultFormatter(
-            self.config["main"]["date_format"],
-            self.config["main"]["time_format"],
-            self.config["main"]["dt_separator"],
+            self.config["date_format"],
+            self.config["time_format"],
+            self.config["dt_separator"],
         )
 
     @cached_property
     def formatter(self):
         return self.formatter_class(
-            self.config["main"]["date_format"],
-            self.config["main"]["time_format"],
-            self.config["main"]["dt_separator"],
+            self.config["date_format"],
+            self.config["time_format"],
+            self.config["dt_separator"],
         )
 
 
@@ -280,7 +269,7 @@ def cli(click_ctx, colour, porcelain, humanize, config):
         )
 
     if humanize is None:  # False means explicitly disabled
-        humanize = ctx.config["main"]["humanize"]
+        humanize = ctx.config["humanize"]
 
     if porcelain:
         ctx.formatter_class = formatters.PorcelainFormatter
@@ -289,7 +278,7 @@ def cli(click_ctx, colour, porcelain, humanize, config):
     else:
         ctx.formatter_class = formatters.DefaultFormatter
 
-    colour = colour or ctx.config["main"]["color"]
+    colour = colour or ctx.config["color"]
     if colour == "always":
         click_ctx.color = True
     elif colour == "never":
@@ -297,28 +286,33 @@ def cli(click_ctx, colour, porcelain, humanize, config):
 
     paths = [
         path
-        for path in glob.iglob(expanduser(ctx.config["main"]["path"]))
-        if isdir(path)
+        for path in glob.iglob(ctx.config["path"])
+        if isdir(path) and not path.endswith("__pycache__")
     ]
     if len(paths) == 0:
-        raise exceptions.NoListsFound(ctx.config["main"]["path"])
+        raise exceptions.NoListsFound(ctx.config["path"])
 
-    ctx.db = Database(paths, ctx.config["main"]["cache_path"])
+    ctx.db = Database(paths, ctx.config["cache_path"])
 
     # Make python actually use LC_TIME, or the user's locale settings
     locale.setlocale(locale.LC_TIME, "")
 
     if not click_ctx.invoked_subcommand:
         invoke_command(
-            click_ctx, ctx.config["main"]["default_command"],
+            click_ctx,
+            ctx.config["default_command"],
         )
 
 
 def invoke_command(click_ctx, command):
-    name, *args = command.split(" ")
+    name, *raw_args = command.split(" ")
     if name not in cli.commands:
-        raise click.ClickException("Invalid setting for [main][default_command]")
-    click_ctx.invoke(cli.commands[command], args)
+        raise click.ClickException("Invalid setting for [default_command]")
+    parser = cli.commands[name].make_parser(click_ctx)
+    opts, args, param_order = parser.parse_args(raw_args)
+    for param in param_order:
+        opts[param.name] = param.handle_parse_result(click_ctx, opts, args)[0]
+    click_ctx.invoke(cli.commands[name], *args, **opts)
 
 
 try:  # pragma: no cover
@@ -336,7 +330,7 @@ except ImportError:
     "--list",
     "-l",
     callback=_validate_list_param,
-    help="The list to create the task in.",
+    help="List in which the task will be saved.",
 )
 @click.option(
     "--read-description",
@@ -356,11 +350,11 @@ def new(ctx, summary, list, todo_properties, read_description, interactive):
 
     todo = Todo(new=True, list=list)
 
-    default_due = ctx.config["main"]["default_due"]
+    default_due = ctx.config["default_due"]
     if default_due:
         todo.due = todo.created_at + timedelta(hours=default_due)
 
-    default_priority = ctx.config["main"]["default_priority"]
+    default_priority = ctx.config["default_priority"]
     if default_priority is not None:
         todo.priority = default_priority
 
@@ -410,7 +404,7 @@ def edit(ctx, id, todo_properties, interactive, raw):
 
     changes = False
     for key, value in todo_properties.items():
-        if value:
+        if value is not None:
             changes = True
             setattr(todo, key, value)
 
@@ -442,7 +436,11 @@ def show(ctx, id):
 @cli.command()
 @pass_ctx
 @click.argument(
-    "todos", nargs=-1, required=True, type=click.IntRange(0), callback=_validate_todos,
+    "todos",
+    nargs=-1,
+    required=True,
+    type=click.IntRange(0),
+    callback=_validate_todos,
 )
 @catch_errors
 def done(ctx, todos):
@@ -456,7 +454,11 @@ def done(ctx, todos):
 @cli.command()
 @pass_ctx
 @click.argument(
-    "todos", nargs=-1, required=True, type=click.IntRange(0), callback=_validate_todos,
+    "todos",
+    nargs=-1,
+    required=True,
+    type=click.IntRange(0),
+    callback=_validate_todos,
 )
 @catch_errors
 def cancel(ctx, todos):
@@ -539,7 +541,7 @@ def move(ctx, list, ids):
     for id in ids:
         todo = ctx.db.todo(id)
         click.echo(ctx.formatter.compact(todo))
-        ctx.db.move(todo, list)
+        ctx.db.move(todo, new_list=list, from_list=todo.list)
 
 
 @cli.command()
@@ -596,7 +598,7 @@ def move(ctx, list, ids):
 @click.option(
     "--status",
     "-s",
-    default=["NEEDS-ACTION", "IN-PROCESS"],
+    default="NEEDS-ACTION,IN-PROCESS",
     callback=validate_status,
     help=(
         "Show only todos with the "
@@ -607,7 +609,7 @@ def move(ctx, list, ids):
 @catch_errors
 def list(ctx, *args, **kwargs):
     """
-    List tasks. Filters any completed or cancelled tasks by default.
+    List tasks (default). Filters any completed or cancelled tasks by default.
 
     If no arguments are provided, all lists will be displayed, and only
     incomplete tasks are show. Otherwise, only todos for the specified list
@@ -623,7 +625,9 @@ def list(ctx, *args, **kwargs):
     The following commands can further filter shown todos, or include those
     omited by default:
     """
-    hide_list = (len([_ for _ in ctx.db.lists()]) == 1) or (len(kwargs["lists"]) == 1)
+    hide_list = (len([_ for _ in ctx.db.lists()]) == 1) or (  # noqa: C416
+        len(kwargs["lists"]) == 1
+    )
 
     todos = ctx.db.todos(**kwargs)
     click.echo(ctx.formatter.compact_multiple(todos, hide_list))
