@@ -4,6 +4,7 @@ class TodomanException(Exception):
 
     Should not be raised directly.
     """
+
     pass
 
 
@@ -11,7 +12,7 @@ class NoSuchTodo(TodomanException):
     EXIT_CODE = 20
 
     def __str__(self):
-        return 'No todo with id {}.'.format(self.args[0])
+        return f"No todo with id {self.args[0]}."
 
 
 class ReadOnlyTodo(TodomanException):
@@ -19,8 +20,9 @@ class ReadOnlyTodo(TodomanException):
 
     def __str__(self):
         return (
-            'Todo is in read-only mode because there are multiple todos in {}.'
-            .format(self.args[0])
+            "Todo is in read-only mode because there are multiple todosin {}.".format(
+                self.args[0]
+            )
         )
 
 
@@ -28,9 +30,8 @@ class NoListsFound(TodomanException):
     EXIT_CODE = 22
 
     def __str__(self):
-        return (
-            'No lists found matching {}, create a directory for a new list.'
-            .format(self.args[0])
+        return "No lists found matching {}, create a directory for a new list.".format(
+            self.args[0]
         )
 
 
@@ -45,4 +46,4 @@ class AlreadyExists(TodomanException):
     EXIT_CODE = 23
 
     def __str__(self):
-        return 'More than one {} has the same identity: {}.'.format(*self.args)
+        return "More than one {} has the same identity: {}.".format(*self.args)
