@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import logging
 import os
 import socket
@@ -8,6 +10,7 @@ from datetime import time
 from datetime import timedelta
 from os.path import normpath
 from os.path import split
+from typing import Iterable
 from uuid import uuid4
 
 import icalendar
@@ -26,7 +29,7 @@ logger = logging.getLogger(name=__name__)
 LOCAL_TIMEZONE = tzlocal()
 
 
-class cached_property:  # noqa
+class cached_property:
     """A read-only @property that is only evaluated once. Only usable on class
     instances' methods.
     """
@@ -56,22 +59,37 @@ class Todo:
     the local system's one.
     """
 
-    def __init__(self, filename=None, mtime=None, new=False, list=None):
+    categories: list[str]
+    completed_at: datetime | None
+    created_at: datetime | None
+    due: date | None
+    dtstamp: datetime | None
+    last_modified: datetime | None
+    related: list[Todo]
+    rrule: str | None
+    start: date | None
+
+    def __init__(
+        self,
+        filename: str = None,
+        mtime: int = None,
+        new: bool = False,
+        list: TodoList = None,
+    ):
         """
         Creates a new todo using `todo` as a source.
 
         :param str filename: The name of the file for this todo. Defaults to
-        the <uid>.ics
+            the <uid>.ics
         :param mtime int: The last modified time for the file backing this
-        Todo.
+            Todo.
         :param bool new: Indicate that a new Todo is being created and should
-        be populated with default values.
-        :param List list: The list to which this Todo belongs.
+            be populated with default values.
+        :param TodoList list: The list to which this Todo belongs.
         """
         self.list = list
         now = datetime.now(LOCAL_TIMEZONE)
-        self.uid = "{}@{}".format(uuid4().hex, socket.gethostname())
-        self.list = list
+        self.uid = f"{uuid4().hex}@{socket.gethostname()}"
 
         if new:
             self.created_at = now
@@ -95,14 +113,14 @@ class Todo:
         self.status = "NEEDS-ACTION"
         self.summary = ""
 
-        self.filename = filename or "{}.ics".format(self.uid)
+        self.filename = filename or f"{self.uid}.ics"
         self.related = []
 
         if os.path.basename(self.filename) != self.filename:
-            raise ValueError("Must not be an absolute path: {}".format(self.filename))
+            raise ValueError(f"Must not be an absolute path: {self.filename}")
         self.mtime = mtime or datetime.now()
 
-    def clone(self):
+    def clone(self) -> Todo:
         """
         Returns a clone of this todo
 
@@ -162,7 +180,7 @@ class Todo:
         "NEEDS-ACTION",
     )
 
-    def __setattr__(self, name, value):
+    def __setattr__(self, name: str, value):
         """Check type and avoid setting fields to None"""
         """when that is not a valid attribue."""
 
@@ -174,7 +192,7 @@ class Todo:
             else:
                 assert isinstance(
                     value, str
-                ), "Got {0} for {1} where str was expected".format(type(value), name)
+                ), f"Got {type(value)} for {name} where str was expected"
 
         if name in Todo.STRING_FIELDS:
             if value is None:
@@ -182,7 +200,7 @@ class Todo:
             else:
                 assert isinstance(
                     value, str
-                ), "Got {0} for {1} where str was expected".format(type(value), name)
+                ), f"Got {type(value)} for {name} where str was expected"
 
         if name in Todo.INT_FIELDS:
             if value is None:
@@ -190,7 +208,7 @@ class Todo:
             else:
                 assert isinstance(
                     value, int
-                ), "Got {0} for {1} where int was expected".format(type(value), name)
+                ), f"Got {type(value)} for {name} where int was expected"
 
         if name in Todo.LIST_FIELDS:
             if value is None:
@@ -198,19 +216,19 @@ class Todo:
             else:
                 assert isinstance(
                     value, list
-                ), "Got {0} for {1} where list was expected".format(type(value), name)
+                ), f"Got {type(value)} for {name} where list was expected"
 
         return object.__setattr__(self, name, v)
 
     @property
-    def is_completed(self):
+    def is_completed(self) -> bool:
         return bool(self.completed_at) or self.status in ("CANCELLED", "COMPLETED")
 
     @property
-    def is_recurring(self):
+    def is_recurring(self) -> bool:
         return bool(self.rrule)
 
-    def _apply_recurrence_to_dt(self, dt):
+    def _apply_recurrence_to_dt(self, dt) -> datetime | None:
         if not dt:
             return None
 
@@ -231,7 +249,7 @@ class Todo:
         # TODO: Push copy's alarms.
         return copy
 
-    def complete(self):
+    def complete(self) -> None:
         """
         Immediately completes this todo
 
@@ -252,10 +270,13 @@ class Todo:
         self.status = "COMPLETED"
 
     @cached_property
-    def path(self):
+    def path(self) -> str:
+        if not self.list:
+            raise ValueError("A todo without a list does not have a path.")
+
         return os.path.join(self.list.path, self.filename)
 
-    def cancel(self):
+    def cancel(self) -> None:
         self.status = "CANCELLED"
 
 
@@ -283,10 +304,10 @@ class VtodoWriter:
         "rrule": "rrule",
     }
 
-    def __init__(self, todo):
+    def __init__(self, todo: Todo):
         self.todo = todo
 
-    def normalize_datetime(self, dt):
+    def normalize_datetime(self, dt: date) -> date:
         """
         Eliminate several differences between dates, times and datetimes which
         are hindering comparison:
@@ -307,7 +328,7 @@ class VtodoWriter:
 
         return dt.astimezone(pytz.UTC)
 
-    def serialize_field(self, name, value):
+    def serialize_field(self, name: str, value):
         if name in Todo.RRULE_FIELDS:
             return icalendar.vRecur.from_ical(value)
         if name in Todo.DATETIME_FIELDS:
@@ -319,9 +340,9 @@ class VtodoWriter:
         if name in Todo.STRING_FIELDS:
             return value
 
-        raise Exception("Unknown field {} serialized.".format(name))
+        raise Exception(f"Unknown field {name} serialized.")
 
-    def set_field(self, name, value):
+    def set_field(self, name: str, value):
         # If serialized value is None:
         self.vtodo.pop(name)
         if value:
@@ -369,8 +390,8 @@ class VtodoWriter:
                 if component.get("uid", None) == self.todo.uid:
                     cal.subcomponents[index] = vtodo
 
-        with AtomicWriter(path, overwrite=True).open() as f:
-            f.write(cal.to_ical().decode("UTF-8"))
+        with AtomicWriter(path, "wb", overwrite=True).open() as f:
+            f.write(cal.to_ical())
 
     def _write_new(self, path):
         vtodo = self.serialize()
@@ -378,10 +399,10 @@ class VtodoWriter:
         c = icalendar.Calendar()
         c.add_component(vtodo)
 
-        with AtomicWriter(path).open() as f:
+        with AtomicWriter(path, "wb").open() as f:
             c.add("prodid", "io.barrera.todoman")
             c.add("version", "2.0")
-            f.write(c.to_ical().decode("UTF-8"))
+            f.write(c.to_ical())
 
         return vtodo
 
@@ -401,7 +422,7 @@ class Cache:
 
     SCHEMA_VERSION = 7
 
-    def __init__(self, path):
+    def __init__(self, path: str):
         self.cache_path = str(path)
         os.makedirs(os.path.dirname(self.cache_path), exist_ok=True)
 
@@ -411,7 +432,7 @@ class Cache:
 
         self.create_tables()
 
-    def save_to_disk(self):
+    def save_to_disk(self) -> None:
         self._conn.commit()
 
     def is_latest_version(self):
@@ -504,7 +525,7 @@ class Cache:
         os.remove(self.cache_path)
         self._conn = None
 
-    def add_list(self, name, path, colour, mtime):
+    def add_list(self, name: str, path: str, colour: str, mtime: int):
         """
         Inserts a new list into the cache.
 
@@ -541,7 +562,7 @@ class Cache:
 
         return self.add_list(name, path, colour, mtime)
 
-    def add_file(self, list_name, path, mtime):
+    def add_file(self, list_name: str, path: str, mtime: int):
         try:
             self._conn.execute(
                 """
@@ -560,7 +581,11 @@ class Cache:
         except sqlite3.IntegrityError as e:
             raise exceptions.AlreadyExists("file", list_name) from e
 
-    def _serialize_datetime(self, todo, field):
+    def _serialize_datetime(
+        self,
+        todo: icalendar.Todo,
+        field: str,
+    ) -> tuple[int | None, bool | None]:
         """
         Serialize a todo field in two value, the first one is the corresponding
         timestamp, the second one is a boolean indicating if the serialized
@@ -581,21 +606,21 @@ class Cache:
             dt = dt.replace(tzinfo=LOCAL_TIMEZONE)
         return dt.timestamp(), is_date
 
-    def _serialize_rrule(self, todo, field):
+    def _serialize_rrule(self, todo, field) -> str | None:
         rrule = todo.get(field)
         if not rrule:
             return None
 
         return rrule.to_ical().decode()
 
-    def _serialize_categories(self, todo, field):
+    def _serialize_categories(self, todo, field) -> str:
         categories = todo.get(field, [])
         if not categories:
             return ""
 
         return ",".join([str(category) for category in categories.cats])
 
-    def add_vtodo(self, todo, file_path, id=None):
+    def add_vtodo(self, todo: icalendar.Todo, file_path: str, id=None) -> int:
         """
         Adds a todo into the cache.
 
@@ -634,7 +659,7 @@ class Cache:
         if start and due:
             start = None if start >= due else start
 
-        params = (
+        params = [
             file_path,
             todo.get("uid"),
             todo.get("summary"),
@@ -654,10 +679,10 @@ class Cache:
             todo.get("sequence", 1),
             self._serialize_datetime(todo, "last-modified")[0],
             self._serialize_rrule(todo, "rrule"),
-        )
+        ]
 
         if id:
-            params = (id,) + params
+            params = [id] + params
             sql = sql.format("id,\n", "?, ")
         else:
             sql = sql.format("", "")
@@ -683,11 +708,8 @@ class Cache:
         due=None,
         start=None,
         startable=False,
-        status=(
-            "NEEDS-ACTION",
-            "IN-PROCESS",
-        ),
-    ):
+        status="NEEDS-ACTION,IN-PROCESS",
+    ) -> Iterable[Todo]:
         """
         Returns filtered cached todos, in a specified order.
 
@@ -718,36 +740,39 @@ class Cache:
         :rtype: generator
         """
         extra_where = []
-        params = []
+        params: list = []
 
         if "ANY" not in status:
+            statuses = status.split(",")
             extra_where.append(
-                "AND status IN ({})".format(", ".join(["?"] * len(status)))
+                "AND (status IN ({}) OR status IS NULL)".format(
+                    ", ".join(["?"] * len(statuses))
+                )
             )
-            params.extend(s.upper() for s in status)
+            params.extend(s.upper() for s in statuses)
 
         if lists:
             lists = [
-                list_.name if isinstance(list_, List) else list_ for list_ in lists
+                list_.name if isinstance(list_, TodoList) else list_ for list_ in lists
             ]
             q = ", ".join(["?"] * len(lists))
-            extra_where.append("AND files.list_name IN ({})".format(q))
+            extra_where.append(f"AND files.list_name IN ({q})")
             params.extend(lists)
         if priority:
             extra_where.append("AND PRIORITY > 0 AND PRIORITY <= ?")
-            params.append("{}".format(priority))
+            params.append(f"{priority}")
         if location:
             extra_where.append("AND location LIKE ?")
-            params.append("%{}%".format(location))
+            params.append(f"%{location}%")
         if category:
             extra_where.append("AND categories LIKE ?")
-            params.append("%{}%".format(category))
+            params.append(f"%{category}%")
         if grep:
             # # requires sqlite with pcre, which won't be available everywhere:
             # extra_where.append('AND summary REGEXP ?')
             # params.append(grep)
             extra_where.append("AND summary LIKE ?")
-            params.append("%{}%".format(grep))
+            params.append(f"%{grep}%")
         if due:
             max_due = (datetime.now() + timedelta(hours=due)).timestamp()
             extra_where.append("AND due IS NOT NULL AND due < ?")
@@ -765,13 +790,13 @@ class Cache:
             extra_where.append("AND (start IS NULL OR start <= ?)")
             params.append(datetime.now().timestamp())
         if sort:
-            order = []
+            order_items = []
             for s in sort:
                 if s.startswith("-"):
-                    order.append(" {} ASC".format(s[1:]))
+                    order_items.append(f" {s[1:]} ASC")
                 else:
-                    order.append(" {} DESC".format(s))
-            order = ",".join(order)
+                    order_items.append(f" {s} DESC")
+            order = ",".join(order_items)
         else:
             order = """
                 completed_at DESC,
@@ -816,25 +841,32 @@ class Cache:
             seen_paths.add(path)
             yield todo
 
-    def _dt_from_db(self, dt, is_date=False):
+    def _datetime_from_db(self, dt) -> datetime | None:
         if dt:
-            val = datetime.fromtimestamp(dt, LOCAL_TIMEZONE)
-            if is_date:
-                val = val.date()
-            return val
+            return datetime.fromtimestamp(dt, LOCAL_TIMEZONE)
         return None
 
-    def _todo_from_db(self, row):
+    def _date_from_db(self, dt, is_date=False) -> date | None:
+        """Deserialise a date (possible datetime)."""
+        if not dt:
+            return dt
+
+        if is_date:
+            return datetime.fromtimestamp(dt, LOCAL_TIMEZONE).date()
+        else:
+            return datetime.fromtimestamp(dt, LOCAL_TIMEZONE)
+
+    def _todo_from_db(self, row: dict) -> Todo:
         todo = Todo()
         todo.id = row["id"]
         todo.uid = row["uid"]
         todo.summary = row["summary"]
-        todo.due = self._dt_from_db(row["due"], row["due_dt"])
-        todo.start = self._dt_from_db(row["start"], row["start_dt"])
+        todo.due = self._date_from_db(row["due"], row["due_dt"])
+        todo.start = self._date_from_db(row["start"], row["start_dt"])
         todo.priority = row["priority"]
-        todo.created_at = self._dt_from_db(row["created_at"])
-        todo.completed_at = self._dt_from_db(row["completed_at"])
-        todo.dtstamp = self._dt_from_db(row["dtstamp"])
+        todo.created_at = self._datetime_from_db(row["created_at"])
+        todo.completed_at = self._datetime_from_db(row["completed_at"])
+        todo.dtstamp = self._datetime_from_db(row["dtstamp"])
         todo.percent_complete = row["percent_complete"]
         todo.status = row["status"]
         todo.description = row["description"]
@@ -846,20 +878,20 @@ class Cache:
         todo.rrule = row["rrule"]
         return todo
 
-    def lists(self):
+    def lists(self) -> Iterable[TodoList]:
         result = self._conn.execute("SELECT * FROM lists")
         for row in result:
-            yield List(
+            yield TodoList(
                 name=row["name"],
                 path=row["path"],
                 colour=row["colour"],
             )
 
     @cached_property
-    def lists_map(self):
+    def lists_map(self) -> dict[str, TodoList]:
         return {list_.name: list_ for list_ in self.lists()}
 
-    def expire_lists(self, paths):
+    def expire_lists(self, paths: dict[str, int]) -> None:
         results = self._conn.execute("SELECT path, name, mtime from lists")
         for result in results:
             if result["path"] not in paths:
@@ -869,10 +901,10 @@ class Cache:
                 if mtime and mtime > result["mtime"]:
                     self.delete_list(result["name"])
 
-    def delete_list(self, name):
+    def delete_list(self, name: str) -> None:
         self._conn.execute("DELETE FROM lists WHERE lists.name = ?", (name,))
 
-    def todo(self, id, read_only=False):
+    def todo(self, id: int, read_only=False) -> Todo:
         # XXX: DON'T USE READ_ONLY
         result = self._conn.execute(
             """
@@ -902,7 +934,7 @@ class Cache:
 
         return self._todo_from_db(result)
 
-    def expire_files(self, paths_to_mtime):
+    def expire_files(self, paths_to_mtime: dict[str, int]) -> None:
         """Remove stale cache entries based on the given fresh data."""
         result = self._conn.execute("SELECT path, mtime FROM files")
         for row in result:
@@ -910,18 +942,18 @@ class Cache:
             if paths_to_mtime.get(path, None) != mtime:
                 self.expire_file(path)
 
-    def expire_file(self, path):
+    def expire_file(self, path: str) -> None:
         self._conn.execute("DELETE FROM files WHERE path = ?", (path,))
 
 
-class List:
-    def __init__(self, name, path, colour=None):
+class TodoList:
+    def __init__(self, name: str, path: str, colour: str = None):
         self.path = path
         self.name = name
         self.colour = colour
 
     @staticmethod
-    def colour_for_path(path):
+    def colour_for_path(path: str) -> str | None:
         try:
             with open(os.path.join(path, "color")) as f:
                 return f.read().strip()
@@ -929,7 +961,7 @@ class List:
             logger.debug("No colour for list %s", path)
 
     @staticmethod
-    def name_for_path(path):
+    def name_for_path(path: str) -> str:
         try:
             with open(os.path.join(path, "displayname")) as f:
                 return f.read().strip()
@@ -937,7 +969,7 @@ class List:
             return split(normpath(path))[1]
 
     @staticmethod
-    def mtime_for_path(path):
+    def mtime_for_path(path: str) -> int:
         colour_file = os.path.join(path, "color")
         display_file = os.path.join(path, "displayname")
 
@@ -952,12 +984,12 @@ class List:
         else:
             return 0
 
-    def __eq__(self, other):
-        if isinstance(other, List):
+    def __eq__(self, other) -> bool:
+        if isinstance(other, TodoList):
             return self.name == other.name
         return object.__eq__(self, other)
 
-    def __str__(self):
+    def __str__(self) -> str:
         return self.name
 
 
@@ -975,8 +1007,8 @@ class Database:
         self.paths = [str(path) for path in paths]
         self.update_cache()
 
-    def update_cache(self):
-        paths = {path: List.mtime_for_path(path) for path in self.paths}
+    def update_cache(self) -> None:
+        paths = {path: TodoList.mtime_for_path(path) for path in self.paths}
         self.cache.expire_lists(paths)
 
         paths_to_mtime = {}
@@ -984,9 +1016,9 @@ class Database:
 
         for path in self.paths:
             list_name = self.cache.add_list(
-                List.name_for_path(path),
+                TodoList.name_for_path(path),
                 path,
-                List.colour_for_path(path),
+                TodoList.colour_for_path(path),
                 paths[path],
             )
             for entry in os.listdir(path):
@@ -1010,8 +1042,7 @@ class Database:
 
             try:
                 with open(entry_path, "rb") as f:
-                    cal = f.read()
-                    cal = icalendar.Calendar.from_ical(cal)
+                    cal = icalendar.Calendar.from_ical(f.read())
                     for component in cal.walk("VTODO"):
                         self.cache.add_vtodo(component, entry_path)
             except Exception:
@@ -1019,27 +1050,29 @@ class Database:
 
         self.cache.save_to_disk()
 
-    def todos(self, **kwargs):
+    def todos(self, **kwargs) -> Iterable[Todo]:
         return self.cache.todos(**kwargs)
 
-    def todo(self, id, **kwargs):
+    def todo(self, id: int, **kwargs) -> Todo:
         return self.cache.todo(id, **kwargs)
 
-    def lists(self):
+    def lists(self) -> Iterable[TodoList]:
         return self.cache.lists()
 
-    def move(self, todo, new_list, from_list=None):
-        from_list = from_list or todo.list
+    def move(self, todo: Todo, new_list: TodoList, from_list: TodoList) -> None:
         orig_path = os.path.join(from_list.path, todo.filename)
         dest_path = os.path.join(new_list.path, todo.filename)
 
         os.rename(orig_path, dest_path)
 
-    def delete(self, todo):
+    def delete(self, todo: Todo) -> None:
+        if not todo.list:
+            raise ValueError("Cannot delete Todo without a list.")
+
         path = os.path.join(todo.list.path, todo.filename)
         os.remove(path)
 
-    def flush(self):
+    def flush(self) -> Iterable[Todo]:
         for todo in self.todos(status=["ANY"]):
             if todo.is_completed:
                 yield todo
@@ -1048,7 +1081,10 @@ class Database:
         self.cache.clear()
         self.cache = None
 
-    def save(self, todo):
+    def save(self, todo: Todo) -> None:
+        if not todo.list:
+            raise ValueError("Cannot save Todo without a list.")
+
         for related in todo.related:
             self.save(related)
 
@@ -1065,6 +1101,6 @@ class Database:
         self.cache.save_to_disk()
 
 
-def _getmtime(path):
+def _getmtime(path: str) -> int:
     stat = os.stat(path)
     return getattr(stat, "st_mtime_ns", stat.st_mtime)
